package com.yeejoin.amos.api.tool.controller;

import com.yeejoin.amos.api.tool.face.model.TemplateCateModel;
import com.yeejoin.amos.api.tool.face.model.TemplateCateTreeModel;
import com.yeejoin.amos.api.tool.face.model.TemplateModel;
import com.yeejoin.amos.api.tool.face.service.TemplateCateService;
import com.yeejoin.amos.api.tool.face.service.TemplateService;
import com.yeejoin.amos.api.tool.face.service.VisualDesignerService;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;
import org.typroject.tyboot.core.foundation.enumeration.UserType;
import org.typroject.tyboot.core.restful.doc.TycloudOperation;
import org.typroject.tyboot.core.restful.utils.ResponseHelper;
import org.typroject.tyboot.core.restful.utils.ResponseModel;

import javax.servlet.http.HttpServletRequest;
import java.util.List;

@CrossOrigin
@RestController
@Api(tags = "模板分类管理Api")
@RequestMapping(value = "/tpl_cate")
public class TemplateCateResource {

    @Autowired
    TemplateCateService templateCateService;
    @Autowired
    TemplateService templateService;
    @Autowired
    VisualDesignerService visualDesignerService;

    /**
     * 新增
     *
     * @return ApiSourceDto
     */
    @TycloudOperation(ApiLevel = UserType.AGENCY,needAuth = false)
    @PostMapping(value = "/save/{sequenceNbr}")
    @ApiOperation(httpMethod = "POST", value = "新增", notes = "新增")
    public ResponseModel<TemplateCateModel> save(@PathVariable Long sequenceNbr, @RequestBody TemplateCateModel templateCateModel){
        templateCateModel.setDesignerType(visualDesignerService.queryBySeq(sequenceNbr).getDesignerType());
        templateCateModel.setIcon("");
        templateCateModel.setIsDel(false);
        templateCateModel = templateCateService.createWithModel(templateCateModel);
        return ResponseHelper.buildResponse(templateCateModel);
    }

    /**
     * 根据sequenceNbr更新
     *
     * @param sequenceNbr 编码
     * @return ApiSourceDto
     */
    @TycloudOperation(ApiLevel = UserType.AGENCY,needAuth = false)
    @PutMapping(value = "/{sequenceNbr}")
    @ApiOperation(httpMethod = "PUT", value = "根据sequenceNbr更新", notes = "根据sequenceNbr更新")
    public ResponseModel<TemplateCateModel> updateBySeqTplCate(@RequestBody TemplateCateModel templateCateModel, @PathVariable(value = "sequenceNbr") Long sequenceNbr){
        templateCateModel.setSequenceNbr(sequenceNbr);
        return ResponseHelper.buildResponse(templateCateService.updateWithModel(templateCateModel));
    }

    /**
     * 根据sequenceNbr删除
     *
     * @param sequenceNbr 编码
     * @return Boolean
     */
    @TycloudOperation(ApiLevel = UserType.AGENCY,needAuth = false)
    @DeleteMapping(value = "/{sequenceNbr}")
    @ApiOperation(httpMethod = "DELETE", value = "根据sequenceNbr删除", notes = "根据sequenceNbr删除")
    public ResponseModel<Boolean> deleteBySequenceNbr(HttpServletRequest request, @PathVariable(value = "sequenceNbr") Long sequenceNbr) {
        List<TemplateModel> templateModels = templateService.queryForTplListByCategorySeq(sequenceNbr);
        for (TemplateModel templateModel : templateModels){
            templateService.removeById(templateModel.getSequenceNbr());
        }
        return ResponseHelper.buildResponse(templateCateService.removeById(sequenceNbr));
    }

    /**
     * 根据sequenceNbr查询
     *
     * @param sequenceNbr 物理主键
     * @return ApiSourceDto
     */
    @TycloudOperation(ApiLevel = UserType.AGENCY,needAuth = false)
    @GetMapping(value = "/{sequenceNbr}")
    @ApiOperation(httpMethod = "GET", value = "根据sequenceNbr查询单个", notes = "根据sequenceNbr查询单个")
    public  ResponseModel<TemplateCateModel> selectOne(@PathVariable Long sequenceNbr){
        return ResponseHelper.buildResponse(templateCateService.queryBySeq(sequenceNbr));
    }

    /**
     * 列表全部数据查询
     *
     * @return List<ApiSourceDto>
     */
    @TycloudOperation(ApiLevel = UserType.AGENCY,needAuth = false)
    @ApiOperation(httpMethod = "GET", value = "列表全部数据查询", notes = "列表全部数据查询")
    @GetMapping(value = "/list")
    public ResponseModel<List<TemplateCateModel>> selectForList(){
        return ResponseHelper.buildResponse(templateCateService.queryForTplCateList());
    }
    /**
     * 列表树形结构查询
     *
     * @return List<ApiSourceDto>
     */
    @TycloudOperation(ApiLevel = UserType.AGENCY,needAuth = false)
    @ApiOperation(httpMethod = "GET", value = "列表树形结构查询", notes = "列表树形结构查询")
    @GetMapping(value = "/tree")
    public ResponseModel<List<TemplateCateTreeModel>> selectForTree(){
        return ResponseHelper.buildResponse(visualDesignerService.getList());
    }

}
