package com.yeejoin.amos.api.tool.face.service;

import com.yeejoin.amos.api.tool.face.model.RelationModel;
import com.yeejoin.amos.api.tool.face.model.RelationTreeModel;
import com.yeejoin.amos.api.tool.face.orm.dao.RelationMapper;
import com.yeejoin.amos.api.tool.face.orm.entity.Relation;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.typroject.tyboot.core.rdbms.service.BaseService;

import java.sql.*;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;

@Service
public class RelationService extends BaseService<RelationModel, Relation, RelationMapper> {

    @Autowired
    DataBaseService dataBaseService;


    /**
     * 删除数据库
     */
    public String deleteDataBase(Connection connection, String agencyCode,String databaseName) throws SQLException {
        try {
            // 加载数据库驱动
            Class.forName("com.mysql.cj.jdbc.Driver");
            if (!connection.isClosed())
                System.out.println("Succeeded connecting to the Database!");
            //通过连接获取relation表树形结构
            List<RelationTreeModel> trees = this.createRelationTree(connection,databaseName);
            for (RelationTreeModel tree : trees) {
                //根据根节点获得根节点相关列表
                List<RelationTreeModel> array = this.createList(tree);
                //根据列表获取sql语句
                System.out.println(this.createSqlByArray(array,agencyCode));
                System.out.println(statementSql(this.createSqlByArray(array, agencyCode),connection));
            }

        } catch (ClassNotFoundException | SQLException e) {
            e.printStackTrace();
        }
        connection.close();
        return null;
}

    /**
     * 执行sql
     */
    public int statementSql(StringBuilder sql, Connection connection) {
        int count = 0;
        try {
            Statement statement = connection.createStatement();
            count = statement.executeUpdate(sql.toString());
            statement.close();
            return count;
        } catch (SQLException e) {
            e.printStackTrace();
        }
        return count;
    }

    /**
     * 获取关系表转化为map
     */
    public List<HashMap<String, String>> queryRelation(String dataBaseName) throws SQLException {
        Connection connection = dataBaseService.connectNewDatabase(this.queryByModel())

        Statement statement = connection.createStatement();
        StringBuilder sql = new StringBuilder("SELECT * FROM ");
        sql.append(dataBaseName).append("_relation ;");
        ResultSet resultSet = statement.executeQuery(sql.toString());
        ResultSetMetaData data = resultSet.getMetaData();
        List<HashMap<String, String>> maps = new ArrayList<>();
        while (resultSet.next()) {
            HashMap<String, String> map = new HashMap<>();
            for (int i = 1; i <= data.getColumnCount(); i++) {
                map.put(data.getColumnName(i), resultSet.getString(data.getColumnName(i)));
            }
            maps.add(map);
        }
        resultSet.close();
        statement.close();
        return maps;
    }

    /**
     * 递归建立树形结构 （算法）
     */
    public List<RelationTreeModel> createTree(List<RelationTreeModel> relationTreeModels, String parentName) {
        List<RelationTreeModel> tree = new ArrayList<>();
        for (RelationTreeModel relationTreeModel : relationTreeModels) {
            if (parentName.equals(relationTreeModel.getParentName())) {
                relationTreeModel.setChildren(createTree(relationTreeModels, relationTreeModel.getName()));
                tree.add(relationTreeModel);
            }
        }
        return tree;
    }

    /**
     * 创建节点列表
     */
    public List<RelationTreeModel> createNodes(List<HashMap<String, String>> maps) {
        List<RelationTreeModel> relationTreeModels = new ArrayList<>();
        for (HashMap<String, String> map : maps) {
            RelationTreeModel model = new RelationTreeModel();
            if (map.get("TABLE1") == null)
                model = new RelationTreeModel(map.get("TABLE2"), map.get("TABLE2_FIELD"), map.get("TABLE1_FIELD"), "", null);
            else
                model = new RelationTreeModel(map.get("TABLE2"), map.get("TABLE2_FIELD"), map.get("TABLE1_FIELD"), map.get("TABLE1"), null);
            relationTreeModels.add(model);
        }
        return relationTreeModels;
    }

    /**
     * 创建关系表树
     */
    public List<RelationTreeModel> createRelationTree(Connection connection,String dataBaseName) throws SQLException {
        List<HashMap<String, String>> maps = this.queryRelation(connection,dataBaseName);
        List<RelationTreeModel> trees = new ArrayList<>();
        trees.addAll(createTree(createNodes(maps), ""));
        return trees;
    }

    /**
     * 根据列表生成sql语句
     */
    public StringBuilder createSqlByArray(List<RelationTreeModel> array, String agencyCode) {
        StringBuilder sql = new StringBuilder();
        if (array.size() == 1) {
            sql.append("DELETE FROM ").append(array.get(0).getName()).append(" WHERE ");
            sql.append(array.get(0).getName()).append(".").append(array.get(0).getField()).append(" = '").append(agencyCode).append("' ;");
        } else {
            sql.append("DELETE ").append(array.get(0).getName());
            for (RelationTreeModel model : array) {
                if (!model.getParentName().equals(""))
                    sql.append(", ").append(model.getName());
            }
            sql.append(" FROM ").append(array.get(0).getName());
            for (RelationTreeModel model : array) {
                if (!model.getParentName().equals("")) {
                    sql.append(" LEFT JOIN ").append(model.getName()).append(" ON ");
                    sql.append(model.getParentName()).append(".").append(model.getParentField()).append(" = ");
                    sql.append(model.getName()).append(".").append(model.getField());
                }
            }
            sql.append(" WHERE ").append(array.get(0).getName()).append(".").append(array.get(0).getField()).append(" = '").append(agencyCode).append("' ;");
        }
        return sql;
    }

    /**
     * 列表节点生成（递归）
     */
    public List<RelationTreeModel> createNodeList(List<RelationTreeModel> tree) {
        List<RelationTreeModel> result = new ArrayList<>();
        for (RelationTreeModel node : tree) {
            RelationTreeModel model = new RelationTreeModel();
            BeanUtils.copyProperties(node, model);
            model.setChildren(null);
            result.add(model);
            result.addAll(createNodeList(node.getChildren()));
        }
        return result;
    }

    /**
     * 树转化对象列表
     */
    public List<RelationTreeModel> createList(RelationTreeModel model) {
        List<RelationTreeModel> result = new ArrayList<>();
        RelationTreeModel temp = new RelationTreeModel();
        BeanUtils.copyProperties(model, temp);
        temp.setChildren(null);
        result.add(temp);
        result.addAll(createNodeList(model.getChildren()));
        return result;
    }

    public RelationModel queryByDbName(String dataBaseName){
        
    }
}
