package com.yeejoin.amos.api.tool.face.service;

import com.alibaba.fastjson.JSON;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.yeejoin.amos.api.tool.face.model.PictureModel;
import com.yeejoin.amos.api.tool.face.model.TemplateModel;
import com.yeejoin.amos.api.tool.face.model.TemplateVoModel;
import com.yeejoin.amos.api.tool.face.orm.dao.TemplateMapper;
import com.yeejoin.amos.api.tool.face.orm.entity.Template;
import com.yeejoin.amos.api.tool.utils.PictureUtil;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;
import org.typroject.tyboot.core.rdbms.service.BaseService;

import java.io.IOException;
import java.io.InputStreamReader;
import java.io.Reader;
import java.util.ArrayList;
import java.util.List;
import java.util.UUID;

@Service
public class TemplateService extends BaseService<TemplateModel, Template, TemplateMapper> {

    @Autowired
    TemplateService templateService;
    @Autowired
    TemplateCateService templateCateService;
    @Autowired
    VisualDesignerService visualDesignerService;

    /**
     * 分页查询
     */
    public Page<TemplateModel> queryForTplPage(Page<TemplateModel> page, Long categorySeq, String displayName) {
        return this.queryForPage(page, null, false, categorySeq, displayName);
    }

    /**
     * 列表查询
     */
    public List<TemplateModel> queryForTplListByCategorySeq(Long categorySeq) {
        return this.queryForList("", false, categorySeq);
    }

    /**
     * 列表查询
     */
    public List<TemplateModel> queryForTplList() {
        return this.queryForList("", false);
    }


    /**
     * TplVoModel转化TplModel
     */
    public TemplateModel convertModel(TemplateVoModel tplVoModel, TemplateModel templateModel) throws NoSuchFieldException, IllegalAccessException {
        BeanUtils.copyProperties(tplVoModel, templateModel, "thumbs", "sketches", "contentStr");
        //判定thumbs为空
        if (tplVoModel.getThumbs().size() != 0)
            templateModel.setThumb(PictureUtil.objectListToUrl(tplVoModel.getThumbs()));
//            templateModel.setThumb(tplVoModel.getThumbs().get(0).getUrl());
//        判定sketch为空
        if (tplVoModel.getSketches().size() != 0)
            templateModel.setSketch(PictureUtil.objectListToUrl(tplVoModel.getSketches()));
//            templateModel.setSketch(tplVoModel.getSketches().get(0).getUrl());
        templateModel.setContent(JSON.parseObject(tplVoModel.getContentStr()));
        return templateModel;
    }

    /**
     * 删除多个模板
     */
    public void deleteByIds(String sequenceNbr) {
        String[] seqs = sequenceNbr.split(",");
        Long[] seq = new Long[seqs.length];
        for (int i = 0; i < seq.length; i++) {
            seq[i] = Long.parseLong(seqs[i]);
            this.removeById(seq[i]);
        }
    }

    /**
     * 读取json文件
     */
    public String readMultipartFile(MultipartFile multipartFile) throws IOException {
        Reader reader = new InputStreamReader(multipartFile.getInputStream(), "utf-8");
        int ch = 0;
        StringBuffer stringBuffer = new StringBuffer();
        while ((ch = reader.read()) != -1) {
            stringBuffer.append((char) ch);
        }
        reader.close();
        return stringBuffer.toString();
    }

    public TemplateVoModel convertVoModel(TemplateModel templateModel){
        TemplateVoModel tplVoModel = new TemplateVoModel();
        BeanUtils.copyProperties(templateModel, tplVoModel);
        if(templateModel.getSketch() != null)
            tplVoModel.setSketches(this.urlToPicModel(templateModel.getSketch()));
        if (templateModel.getThumb() != null)
            tplVoModel.setThumbs(this.urlToPicModel(templateModel.getThumb()));
        System.out.println(tplVoModel.getThumbs());
        return tplVoModel;
    }

    public String getUid(){
        UUID uuid= UUID.randomUUID();
        String str = uuid.toString();
        return str.replace("-", "");
    }

    public List<PictureModel> urlToPicModel(String url){
        List<PictureModel> pictureModels = new ArrayList<>();
        PictureModel pictureModel = new PictureModel("", this.getUid(), url, "done");
        pictureModels.add(pictureModel);
        return pictureModels;
    }


}
